//-*-c++-*-
/***************************************************************************
 *   Copyright (C) 2003 by Fred Schaettgen                                 *
 *   kbluetoothd@schaettgen.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <kprocess.h>
#include "trayicon.h"

using namespace KBluetooth;
using namespace std;

TrayIcon::TrayIcon(KBluetoothApp* app)
{
    iconConnectingBlinkState = false;
    this->helpMenu = new KHelpMenu(this, KApplication::kApplication()->aboutData());
    this->app = app;
    //iconState = IDLE;
    KConfig *config = KGlobal::config();
    config->setGroup("UI");
    alwaysVisible = config->readBoolEntry("alwaysVisible", true);
    acceptClose = true;
    blinkTimer = new QTimer(this);
    kdDebug() << "geschafft" << endl;
    connect(blinkTimer, SIGNAL(timeout()), this, SLOT(updateIcon()));
    
  // KBlueMon
    kbluemonAction = new KAction(i18n("KBlue&Mon"),
        KGlobal::iconLoader()->loadIcon("info", KIcon::Small, 16), 
	KShortcut::null(), this, "kbluemon");
    connect(kbluemonAction, SIGNAL(activated()), this, SLOT(slotkbluemonitor()));
    kbluemonAction->plug(contextMenu());

  // KBlueLock
    kbluelockAction = new KAction(i18n("KBlue&Lock"),
        KGlobal::iconLoader()->loadIcon("lock", KIcon::Small, 16), 
	KShortcut::null(), this, "kbluelock");
    connect(kbluelockAction, SIGNAL(activated()), this, SLOT(slotkbluelock()));
    kbluelockAction->plug(contextMenu());

    // Configuration menu
    configActionMenu = new KActionMenu(i18n("&Configuration"),
        KGlobal::iconLoader()->loadIcon("configure", KIcon::Small, 16), this, "config_menu");
    configActionMenu->plug(contextMenu());

    // Help menu
    helpActionMenu = new KActionMenu(i18n("&Help"),
        KGlobal::iconLoader()->loadIcon("help", KIcon::Small, 16), this, "help_menu");
    helpActionMenu->plug(contextMenu());

    // Menu entries:
    // -------------

    // Inputdevice configuration
    inputConfigAction = new KAction(i18n("&Input Devices..."),
        KGlobal::iconLoader()->loadIcon("configure", KIcon::Small, 16),
        KShortcut::null(), this, "input_config");
    connect(inputConfigAction, SIGNAL(activated()), this, SLOT(slotInputConfig()));
    configActionMenu->insert(inputConfigAction);

    // Service configuration
    serviceConfigAction = new KAction(i18n("Configure &Services..."),
        KGlobal::iconLoader()->loadIcon("configure", KIcon::Small, 16),
        KShortcut::null(), this, "service_config");
    connect(serviceConfigAction, SIGNAL(activated()), this, SLOT(slotServiceConfig()));
    configActionMenu->insert(serviceConfigAction);

    // Bluetooth Device configuration
    deviceConfigAction = new KAction(i18n("&Devices..."),
        KGlobal::iconLoader()->loadIcon("configure", KIcon::Small, 16),
        KShortcut::null(), this, "device_config");
    connect(deviceConfigAction, SIGNAL(activated()), this, SLOT(slotDeviceConfig()));
    configActionMenu->insert(deviceConfigAction);

    // Paired device configuration
    pairedConfigAction = new KAction(i18n("&Paired Devices..."),
        KGlobal::iconLoader()->loadIcon("configure", KIcon::Small, 16),
        KShortcut::null(), this, "paired_config");
    connect(pairedConfigAction, SIGNAL(activated()), this, SLOT(slotPairedConfig()));
    configActionMenu->insert(pairedConfigAction);

    // configure notifications
    /*
    KAction* notificationAction = KStdAction::configureNotifications(this,
        SLOT(slotConfigureNotifications()), actionCollection());
    configActionMenu->insert(notificationAction);
    */

    // Always-visible switch
    showIconAction = new KToggleAction(this, "always_visible");
    connect(showIconAction, SIGNAL(toggled(bool)), this, SLOT(slotShowIconToggled(bool)));
    showIconAction->setChecked(alwaysVisible);
    showIconAction->setText(i18n("Always &Visible"));
    configActionMenu->insert(showIconAction);

    // show documentation
    KAction* showHelpAction = KStdAction::help(this,
        SLOT(slotShowHelp()), actionCollection());
    helpActionMenu->insert(showHelpAction);

    // Report bug menu item
    /*
    KAction* reportBugAction = KStdAction::reportBug(this,
        SLOT(slotReportBug()), actionCollection());
    helpActionMenu->insert(reportBugAction);
    */

    // "About" menu item
    KAction* aboutAction = KStdAction::aboutApp(this,
        SLOT(slotAbout()), actionCollection());
    helpActionMenu->insert(aboutAction);

    KPixmap logoPixmap = KGlobal::iconLoader()->loadIcon(
        "kdebluetooth", KIcon::Small, 22);
    iconIdle = logoPixmap;
    iconConnected = logoPixmap;
    iconNoAdapter = logoPixmap;
    KPixmapEffect::toGray(iconIdle);
    KPixmapEffect::fade(iconIdle, 0.3, QColor(255,255,255));
    KPixmapEffect::toGray(iconNoAdapter);
    KPixmapEffect::fade(iconNoAdapter, 0.5, QColor(128,128,128));
    updateIcon();

    connect(this, SIGNAL(quitSelected()),
        this, SLOT(slotQuitSelected()));

}

TrayIcon::~TrayIcon()
{
}

void TrayIcon::setAlwaysShowIcon(bool state) {
    showIconAction->setChecked(state);
    slotShowIconToggled(state);
}

void TrayIcon::slotShowIconToggled(bool state)
{
    KConfig *config = KGlobal::config();
    config->setGroup("UI");
    config->writeEntry("alwaysVisible", state);
    config->sync();
    alwaysVisible = state;
    if (state == false) {
        KMessageBox::information(this,
            i18n("KBluetooth will run in hidden mode now and only show up when there is \
a connection. To enable the permanent tray icon start KBluetooth again."),
            i18n("Hidden Mode"), "hiddenmode_messagebox");
    }
    updateIcon();
}

void TrayIcon::updateIcon()
{
    QString oldText = QToolTip::textFor(this);
    QString newText = "";
    manager = app->manager;
    Adapter::ConnectionList connList;
	if (alwaysVisible) {
	    if (manager->listAdapters().count() > 0) {
	       setPixmap(iconIdle);
	    }
	    else {
		setPixmap(iconNoAdapter);
	    }
	    show();
	} else {
	    hide();
	}
	//showMonitorAction->setEnabled(false);
	newText = i18n("Not connected");

	Manager::AdapterList list = manager->listAdapters();
	Manager::AdapterList::iterator it;
	for (it = list.begin(); it != list.end(); ++it) {
		if (*it == manager->defaultAdapter()) {
			adapter = new Adapter((*it), app->conn);
			connList = adapter->listConnections();
		 } else {
			Adapter a((*it), app->conn);
			connList = a.listConnections();
		}
		
		if (connList.size()) {
			setPixmap(iconConnected);
			newText = i18n("Connected");
			break;
		}

	}

	/*
        bool connecting = false;
        if (connecting) {
            if (!blinkTimer->isActive()) 
		    blinkTimer->start(250);

            iconConnectingBlinkState = !iconConnectingBlinkState;

            if (iconConnectingBlinkState) {
                setPixmap(iconConnected);
            }
            else {
                setPixmap(iconIdle);
            }
        }
        else {
            blinkTimer->stop();
            setPixmap(iconConnected);
        }
	*/



        show();
        //showMonitorAction->setEnabled(true);
	/*
        QString toolTipText = QString("<b></b>")+i18n("Connected to ");
        for (conIt = connections.begin(); conIt != connections.end(); ++conIt) {
            QString devName;
            if (connections.size() > 1) {
                toolTipText += "<br/>";
            }
            if ((devName = app->deviceNameCache->getCachedDeviceName(conIt->first))
                != QString::null)
            {
                toolTipText += QString(
                    "<b>%1</b> [%2]").arg(devName).arg(QString(conIt->first));
            }
            else {
                toolTipText += QString("[<b>%1</b>]").arg(QString(conIt->first));
            }
        }

        toolTipText += "</p>";
        newText = toolTipText;
	*/

    if (oldText != newText) {
        QToolTip::remove(this);
        QToolTip::add(this, newText);
    }

}

void TrayIcon::slotServiceConfig()
{
    // We have to run the control center module with
    // KProcessInheritSocket, because otherwise the kcm
    // would inherit all open sockets and cause problems
    // when kbluetooth::MetaServer::reload is called somewhere
	/*
    KProcessInheritSocket process(0);
    process << "kcmshell" << "kcm_kbluetooth";
    if (!process.start(KProcess::DontCare)) {
        KMessageBox::information(this,
            i18n("Could not execute the kbluetooth Control Center module."),
            i18n("KBluetooth"));

    }
    */
}

void TrayIcon::slotPairedConfig()
{
    KProcess process(0);
//    process << "kdesu" << "--nonewdcop" << "-i" << "kdebluetooth" << "kcmshell" << "kcm_btpaired";
    process << "kcmshell" << "kcm_btpaired";

    if (!process.start(KProcess::DontCare)) {
        KMessageBox::information(this,
            i18n("Could not execute the btpaired Control Center module."),
            i18n("KBluetooth"));

    }
}

void TrayIcon::slotDeviceConfig()
{
    kdDebug() << __func__ << "()" << endl;
    deviceConfDialog = new DeviceConfig(app);
    deviceConfDialog->show();
}

void TrayIcon::slotInputConfig()
{
    kdDebug() << __func__ << "()" << endl;
    KProcess process(0);

    process << "kinputwizard";

    if (!process.start(KProcess::DontCare)) {
        KMessageBox::information(this,
            i18n("Could not execute kinputwizard."),
            i18n("KBluetooth"));
    }	
	
}
void TrayIcon::slotkbluemonitor()
{
    kdDebug() << __func__ << "()" << endl;
    KProcess process(0);

    process << "kbluemon";

    if (!process.start(KProcess::DontCare)) {
        KMessageBox::information(this,
            i18n("Could not execute KBlueMon."),
            i18n("KBluetooth"));
    }	
}
void TrayIcon::slotkbluelock()
{
    kdDebug() << __func__ << "()" << endl;
    KProcess process(0);

    process << "kbluelock";

    if (!process.start(KProcess::DontCare)) {
        KMessageBox::information(this,
            i18n("Could not execute KBlueLock."),
            i18n("KBluetooth"));
   }
}

void TrayIcon::mousePressEvent(QMouseEvent *e)
{
    if (e->button() == QMouseEvent::LeftButton) {
        e->accept();
        KProcess proc;
        proc << "kfmclient" << "openURL" << "bluetooth:/";
        proc.start(KProcess::DontCare);
    }
    else {
        KSystemTray::mousePressEvent(e);
    }
}

void TrayIcon::slotQuitSelected()
{
    // Ask if the user want to simply quit or disable
    // automatic start of kbluetooth
    int autoStart = KMessageBox::questionYesNoCancel( 0,
                                                      i18n("Should KBluetooth still be restarted when you login?"),
                                                      i18n("Automatically Start KBluetooth?"),i18n("Start"), i18n("Do Not Start") );
    acceptClose = true;
    KConfig *config = KGlobal::config();
    config->setGroup("General");
    if (autoStart == KMessageBox::Yes) {
        config->writeEntry("AutoStart", true);
    }
    else if (autoStart == KMessageBox::No) {
        config->writeEntry("AutoStart", false);
    } else {
        acceptClose = false;
        return;
    }

    config->sync();
}


void TrayIcon::slotShowHelp()
{
    // TODO: This is surely not the correct way to jump to the
    // right chapter. Do I really have to mention the html-file,
    // or is the id enough?
    KApplication::kApplication()->invokeHelp("",
        "kdebluetooth/components.html#components.kbluetooth");
}

QString TrayIcon::localAndEnglish(const QCString& s)
{
    if (QString(s) != i18n(s)) {
        return QString("%1 (\"%2\")").arg(i18n(s)).arg(s);
    }
    else {
        return s;
    }
}

#if 0 
// TODO : make use of KNofityClient 
void TrayIcon::slotConnectionComplete(int status, QString addr)
{
    KNotifyClient::event(
#if (QT_VERSION >= 0x030200)
        KApplication::kApplication()->mainWidget()->winId(),
#endif
        "ConnectionError",
        i18n("<i>Bluetooth Monitor</i><br/> Problem connecting with <b>%1</b>:")
            .arg(name)+"<br/>"+errorMessage);
}
#endif

#include "trayicon.moc"
